/*! Tablerise - v0.1.0 - 2012-12-28
* https://github.com/abner/tablerise
* Copyright (c) 2012 Abner Carleto; Licensed MIT */

;(function ( $, window, undefined ) {

  // Create the defaults once
  "use strict";
  var pluginName = 'tablerise',
      document = window.document,
      defaults = {
        selectable: true,
        selectableClass: 'row-selected',
        selected: null, // action executed on selected line
        deselected: null, // action executed on deselected line
        multipleSelect: false
      };

  // The actual plugin constructor
  function Plugin( element, options ) {
    this.element = element;

    // set options
    this.options = $.extend( {}, defaults, options) ;

    this._defaults = defaults;
    this._name = pluginName;

    this.init();
  }

  Plugin.prototype.init = function () {
    this.selectable();
  };

  // This method creates events on click in table row and add ou remove css-class
  // define in options.selectableClass
  Plugin.prototype.selectable = function() {
    var that = this;
    if (this.options.selectable) {
      $(this.element).undelegate('tbody > tr', 'click').delegate('tbody > tr', 'click', function(event) {
        if ( $(this).hasClass(that.options.selectableClass) ) {
          that.selectableDeselect(this, event);
        } else {
          that.selectableSelect(this, event);
        }
      });
    }
  };

  // Action executed when table row is selected
  Plugin.prototype.selectableSelect = function(element, event) {
    if (!this.options.multipleSelect) {
      $(element).parents('tbody').find('.' + this.options.selectableClass).removeClass(this.options.selectableClass);
    }
    $(element).addClass(this.options.selectableClass);
    if (typeof this.options.selected === 'function') {
      this.options.selected.apply(element, [event]);
    }
  };

  // Action executed when table row is deselected
  Plugin.prototype.selectableDeselect = function(element, event) {
    $(element).removeClass(this.options.selectableClass);
    if (typeof this.options.deselected === 'function') {
      this.options.deselected.apply(element, [event]);
    }
  };

  // A really lightweight plugin wrapper around the constructor, 
  // preventing against multiple instantiations
  $.fn[pluginName] = function ( options ) {
    return this.each(function () {
      if (!$.data(this, 'plugin_' + pluginName)) {
        $.data(this, 'plugin_' + pluginName, new Plugin( this, options ));
      }
    });
  };

}(jQuery, window));